from pdbfixer import PDBFixer
from simtk.openmm.app import PDBFile
from subprocess import call
import sys

# usage: python get_pMHC_pdb.py <pdb code>
# assumes pdb code is of a peptide-MHC structure
# adds missing atoms/residues, removes all waters and ions, labels chains as A,B,C where chain C is the peptide
def main(args):
    
    pdbcode = args[0]

    if len(pdbcode) != 4:
        print("Please enter a correct 4 letter pdbid")
        sys.exit(0)

    call("pdbfixer --pdbid " + pdbcode + " --output=temp.pdb --add-atoms=heavy --keep-heterogens=none", shell=True)
    fixer = PDBFixer(filename="temp.pdb")
    found_A = False
    found_B = False
    found_C = False

    num_chains = len(list(fixer.topology.chains()))
    chains = fixer.topology.chains()
    chains_to_remove = []
    for i, c in enumerate(chains):
        num_residues = len(list(c.residues()))
        if num_residues > 250:
            if not found_A:
                found_A = True
                c.id = "A"
            else:
                c.id = "Z"
                chains_to_remove.append(i)
        elif num_residues > 50 and num_residues < 150:
            if not found_B:
                found_B = True
                c.id = "B"
            else:
                c.id = "Z"
                chains_to_remove.append(i)
        elif num_residues <= 15:
            if not found_C:
                found_C = True
                c.id = "C"
            else:
                c.id = "Z"
                chains_to_remove.append(i)
        else:
            c.id = "Z"
            chains_to_remove.append(i)
            #print "ERROR: Found chains with weird number of residues:", num_residues
            #sys.exit(0)

    fixer.removeChains(chains_to_remove)

    chains = fixer.topology.chains()
    chain_lengths = []
    for c in chains:
        num_residues = len(list(c.residues()))
        chain_lengths.append(num_residues)

    PDBFile.writeFile(fixer.topology, fixer.positions, open(pdbcode + ".pdb", 'w'))
    call(["rm temp.pdb"], shell=True)

    if chain_lengths[1] < chain_lengths[2]:
        call(["grep \"[A-Z] B  \" " + pdbcode + ".pdb > temp.pdb"], shell=True)
        call(["sed -i \"/[A-Z] B  /d\" " + pdbcode + ".pdb"], shell=True)
        call(["sed -i \"/END/d\" " + pdbcode + ".pdb"], shell=True)
        call(["sed -i \"/CONECT/d\" " + pdbcode + ".pdb"], shell=True)
        call(["less temp.pdb >> " + pdbcode + ".pdb"], shell=True)
        fixer = PDBFixer(filename=pdbcode + ".pdb") 
        PDBFile.writeFile(fixer.topology, fixer.positions, open(pdbcode + ".pdb", 'w'))
        call(["rm temp.pdb"], shell=True)

if __name__ == "__main__":
    main(sys.argv[1:])

    







#!/usr/bin/python

#       Sgourakis Lab
#   Author: Santrupti Nerli
#   Date: August 22, 2018
#   Email: snerli@ucsc.edu
#

'''

CHAPERONE class contains all the necessary functionalities required to fetch
necessary chaperone sequences and their respective headers.

'''

# Load the Rosetta commands for use in the Python shell
from database.chaperone_sequences import chaperone_sequences
from input_output.input.header_readers import HEADER_READER

# inherit from HEADER_READER, a generic class that reads
# headers from a file
class CHAPERONE(HEADER_READER):

    # constructor
    def __init__(self, filename):
        super().__init__(filename)
        super().read_headers()

    # getter method
    def get_sequence(self, header):
        return chaperone_sequences[header]



# pymol -qc mutate.py pdb selection new_residue
# example: pymol -qc mutate.py 0.pdb C/1/ ALA 0_mutated.pdb

# run for all the confs in the ensemble
# append all to one using mdconvert

from pymol import cmd
import sys

pdb, selection, mutant, output_name = sys.argv[-4:]
print("file:", pdb)
print("selection:", selection)
print("mutating to:", mutant)
print("output name:", output_name)

cmd.load(pdb, "file")

cmd.wizard("mutagenesis")
cmd.refresh_wizard()
cmd.get_wizard().do_select(selection)
cmd.get_wizard().set_mode(mutant)
cmd.get_wizard().apply()
cmd.set_wizard()

cmd.save(output_name, "file")



#!/usr/bin/python

#       Sgourakis Lab
#   Author: Santrupti Nerli
#   Date: January 31, 2018
#   Email: snerli@ucsc.edu
#

'''

JOB_DISTRIBUTOR class contains all the necessary functionalities required
parallelize RosettaMHC method using MPI (message passing interface).

'''

# Load the Rosetta commands for use in the Python shell
from pyrosetta import *

# import required libraries
from mpi4py import MPI
import pickle

class JOB_DISTRIBUTOR:

    # class members
    rank = -1 # processor number
    comm = None # MPI communicator object
    size = -1 # number of process selected during mpi execution (ex. mpiexec -np 3 implies size = 3)
    done = False # this parameter is used as a flag for executing a function by a single processor only

    # constructor
    def __init__(self):
        self.comm = MPI.COMM_WORLD
        self.rank = self.comm.Get_rank()
        self.size = self.comm.Get_size()

    # method to sync all the processors
    def sync(self):
        self.comm.barrier()

    # method to broadcast data from 0th process to other processes
    def broadcast(self, data):
        return self.comm.bcast(data, root = 0)

    # set the done parameter and broadcast to other processes that
    # the task is done
    def set_all_dones(self):
        self.done = self.comm.bcast(self.done, root = 0)

    # getter methods
    def get_communicator(self):
        return self.comm

    def get_rank(self):
        return self.rank

    # method to make only one single process execute a given function
    # currently this method does not guarantee that the first process executing this
    # is rank 0
    def perform_single_operation(self, fun):
        if not self.done:
            self.sync()
            ret_value = ""
            if self.rank == 0:
                print ("In rank: ", self.rank, " .Performing template preparation.")
                ret_value = fun()
                self.done = True
                self.set_all_dones()

            # broadcast the ret_val to other ranks
            ret_value = self.comm.bcast(ret_value, root=0)
            print ("Reached here. Return value is: ", ret_value)
            return ret_value

    # method to perform MPI based job distribution across multiple processes
    # this is the same code from https://graylab.jhu.edu/Sergey/PyRosetta/000/rosetta/#rosetta.PyJobDistributor
    # and modified a little
    def MPIJobDistributor(self, njobs, fun):
        print ("No. of jobs received: ", njobs )

        myjobs = []

        if self.rank == 0:
            jobs = list(range(njobs))
            jobs.extend( [None]*(self.size - njobs % self.size) )
            n = len(jobs)/self.size
            for i in range(self.size):
                queue = []  # list of jobs for individual cpu
                for j in range(int(n)):
                    queue.append(jobs[j*self.size+i])

                if( i == 0 ):
                    myjobs = queue
                else:
                    # now sending the queue to the process
                    print ('Sending %s to node %s' % (queue, i) )
                    self.comm.send(queue, dest=i)
        else:
            # getting decoy lists
            myjobs = self.comm.recv(source=0)

        print('Node %s, got queue:%s' % (self.rank, myjobs) )

        for j in myjobs:
            if j is not None:
                fun(j)

    # method to perform simple job distribution
    # divide the runs into each core equally
    def SimpleMPIJobDistributor(self, njobs, fun):
        print ("No. of jobs received: ", njobs )
        print ("I am in rank: ", self.rank )
        for i in range(njobs):
            #if i%self.size != self.rank: continue
            print ("Performing task ", i, " now in rank: ", self.rank )
            fun(i)



#!/usr/bin/python

#       Sgourakis Lab
#   Author: Santrupti Nerli
#   Date: January 26, 2017
#   Email: snerli@ucsc.edu
#

# import other required libraries
import os
import sys
import subprocess
from collections import defaultdict

'''

FASTA class contains all the necessary functionalities required to read
and write to fasta sequence files.

'''

class FASTA:

    # class members
    filename = "" # input file name
    sequences = defaultdict(dict) # sequences read from the fasta file name

    # constructor
    def __init__(self, filename):
        self.filename = filename
        self.sequences = defaultdict(dict)

    # method to get the sequence headers
    def get_headers(self):
        if not self.sequences:
            self.sequences['none'] = ''
        return self.sequences.keys()

    # method to get sequences read from fasta file
    def get_sequences(self):
        return self.sequences

    # method to get a sequence given its header
    def get_sequence(self, header):
        return self.sequences[header]

    # method to read the fasta file
    # populates headers and sequences
    def read(self):
        if self.filename != None:
            fasta_file_handler = open(self.filename, "r")
            for line in fasta_file_handler:
                line = line.rstrip()
                if ">" in line:
                    header = "".join(list(line)[1:])
                else:
                    self.sequences[header] = line
            fasta_file_handler.close()

    # method to write to fasta file
    def write(self, header, text):
        fasta_file_handler = open(self.filename, "w")
        fasta_file_handler.write(">"+header+"\n")
        fasta_file_handler.write(text+"\n")
        fasta_file_handler.close()



from setuptools import setup,find_packages

with open('./README.md', 'r') as f:
    readme = f.read()
with open('./requirements.txt') as f:
    requires_packages  = f.read().splitlines()

setup(
    name='settmhc',
    version='1.0.0',
    description='A peptide-MHC binding predictor based on sequence-structure information',
    author='Chen Wenfan',
    author_email='wenfanchan@zju.edu.cn',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'settmhc = settmhc.settmhc_main:main',
            'pshm = settmhc.pshm:main',
            'pmhc-model = settmhc.structure_modeling:structure_generator', ],
    },
    long_description=readme,
    long_description_content_type="text/markdown",
    install_requires = requires_packages
)



import torch
import os.path as osp
import pandas as pd
import re
from tqdm import tqdm
from .utils import Process
from .structure_modeling import make_structure
from .pshm import parse_pdbdata

DEFAULT_DIR = osp.dirname(osp.realpath(__file__))
re_hla = re.compile(r'(HLA\-[ABC])(.*)')
hla_table = pd.read_table(osp.join(DEFAULT_DIR, 'MHC_pseudo.dat'))
hla_dict = {}
for tup in hla_table.itertuples():
    hla = tup.HLA
    seq = tup.sequence
    if re_hla.match(hla):
        hla = re_hla.match(hla).group(1) + '*' + re_hla.match(hla).group(2)
        hla_dict[hla] = seq


def make_annotations(nums):
    return ['pmhc'+str(i).zfill(len(str(nums))) for i in range(1, nums+1)]


def make_sequence_tensor(peptide, hla):
    hla_seq = hla_dict[hla]
    seq = hla_seq + peptide
    process = Process(seqs=[seq])
    return process.seqs_aa2idx()


def make_data_from_pdb(peptide, hla, pdb_file, protein='A', ligand='C'):
    cnn_data = torch.Tensor(parse_pdbdata(
        pdb_file, protein, ligand)).reshape((1, 16, 360))
    rnn_data = make_sequence_tensor(peptide, hla)
    return cnn_data, rnn_data


def make_data_from_csv(df, sequence_only=False):
    peptides = df['peptide']
    hlas = df['HLA']
    annos = df['Annotation']
    cnn_datas = torch.zeros((len(peptides), 1, 16, 360))
    rnn_datas = torch.zeros((len(peptides), 49)).long()
    skipped_idx = []
    if not sequence_only:
        i = 0
        pbar = tqdm(total=len(df), desc="Preprocessing")
        for peptide, hla, anno in zip(peptides, hlas, annos):
            output_pdb = make_structure(
                peptide, hla, anno, structure_dir='structures')
            if output_pdb is not None:
                cnn_data, rnn_data = make_data_from_pdb(
                    peptide, hla, output_pdb)
                cnn_datas[i] = cnn_data
                rnn_datas[i] = rnn_data
            else:
                skipped_idx.append(i)
            i += 1
            pbar.update(1)
        pbar.close()
    else:
        i = 0
        pbar = tqdm(total=len(df), desc="Preprocessing")
        for peptide, hla, anno in zip(peptides, hlas, annos):
            rnn_datas[i] = make_sequence_tensor(peptide, hla)
            i += 1
            pbar.update(1)
        pbar.close()
    return cnn_datas, rnn_datas, skipped_idx



#!/usr/bin/python

#       Sgourakis Lab
#   Author: Santrupti Nerli
#   Date: January 26, 2017
#   Email: snerli@ucsc.edu
#

# Load the Rosetta commands for use in the Python shell
from pyrosetta import *
from pyrosetta.toolbox import cleanATOM

# import rosetta files
from pyrosetta.rosetta.protocols.relax import *
from pyrosetta.rosetta.core.scoring import *
from idealize_relax.idealize import IDEALIZE
from idealize_relax.relax import RELAX

# additional bio library
from Bio.PDB import *

'''

TEMPLATE class contains all the necessary functionalities required to
perform prepare the template structure for threading.

'''


class TEMPLATE:

    # class members
    template_pdb = None # store template pdb file name
    template_pose = None # store the template pose

    # constructor
    def __init__(self, pdb):
        self.template_pdb = pdb

    # method to return pose
    def get_pose(self):
        return self.template_pose

    # method to return sequence of the template
    def get_sequence(self):
        return self.template_pose.sequence()

    # method to return name of the template pdb file
    def get_pdb(self):
        return self.template_pdb

    # method to load pose from a pdb file
    def get_pose_from_pdb(self):
        if self.template_pose == None:
            self.template_pose = pose_from_pdb(self.template_pdb)
        return self.template_pose

    # method to return pdb name fields joined with dot
    # some of the naming convention we use
    def get_name(self):
        template_pdb_fields = self.template_pdb.split(".")
        template_pdb_name = ".".join(template_pdb_fields[0:len(template_pdb_fields)-1])
        return template_pdb_name

    # method to get the striped pdb name or file name without the .pdb extension
    def get_stripped_name(self):
        template_pdb_name = os.path.basename(self.template_pdb)
        return template_pdb_name.split('.')[0]

    # methot to return absolute path of the file where template is present
    def get_template_path(self):
        return os.path.abspath(os.path.dirname(self.template_pdb))+"/"

    # method to trim the pdb if specified
    def trim_pdb(self, mhc_chain, peptide_chain, mhc_trim_length):
        p = PDBParser()
        s = p.get_structure('X', self.template_pdb)
        class ResSelect(Select):
            def accept_residue(self, res):
                if res.id[1] >= mhc_trim_length and res.parent.id == mhc_chain:
                    return False
                if res.parent.id != mhc_chain and res.parent.id != peptide_chain:
                    return False
                else:
                    return True
        io = PDBIO()
        io.set_structure(s)
        self.append_cropped_pdb()
        io.save(self.template_pdb, ResSelect())

    # method to save pdb file
    def save_pdb(self):
        self.template_pose.dump_pdb(self.template_pdb)

    # method to append cleaned pdb after cleaning the template
    def append_clean_pdb(self):
        template_pdb_name = self.get_name()
        self.template_pdb = template_pdb_name + ".clean.pdb"

    # method to append cropped pdb after cropping the template structure
    def append_cropped_pdb(self):
        template_pdb_name = self.get_name()
        self.template_pdb = template_pdb_name + ".cropped.pdb"

    # method to clean, crop, idealize or relax pdb file
    def treat_template_structure(self, mhc_chain, peptide_chain, trim_mhc, mhc_trim_length, idealize_relax = False):
        cleanATOM(self.template_pdb)
        self.append_clean_pdb()
        if trim_mhc:
            self.trim_pdb(mhc_chain, peptide_chain, mhc_trim_length)
        self.template_pose = pose_from_pdb(self.template_pdb)
        if idealize_relax == True:
            template_pose = self.template_pose
            self.template_pose = IDEALIZE().idealize_pdb(template_pose)
            self.template_pose = RELAX().relax_pdb(template_pose)
        self.save_pdb()



from argparse import Namespace
import numpy as np
import torch
import torch.nn as nn
import pytorch_lightning as pl
from torchmetrics import Accuracy
import math
from .utils import get_linear_schedule_with_warmup


class Positional_wise_input(nn.Module):
    def __init__(self, embed_dim, max_length, gpu=0):

        super().__init__()
        self.encoding = []
        for pos in range(max_length):
            ls = [pos / (10000.0 ** (i // 2 * 2.0 / embed_dim))
                  for i in range(embed_dim)]
            self.encoding.append(ls)
        self.encoding = np.array(self.encoding)
        self.encoding[:, 0::2] = np.sin(self.encoding[:, 0::2])
        self.encoding[:, 1::2] = np.cos(self.encoding[:, 1::2])
        self.encoding = torch.Tensor(self.encoding)

    def forward(self, x: torch.Tensor):
        # x: [batch, embed_dim, width]
        x = x.permute(0, 2, 1)
        encoding = self.encoding.to(x.device)
        out = x + encoding
        return out


class CNN_model(nn.Module):
    def __init__(self, config):
        super().__init__()
        self.config = config
        # out_size = (n+2p-f)/s + 1
        acti_func = config['activation']
        conv1_size = config['conv1']
        conv2_size = config['conv2']
        conv3_size = config['conv3']
        kernel1_size = config['kernel1']
        kernel2_size = config['kernel2']
        kernel3_size = config['kernel3']
        dropout = config['dropout']
        width = config['width']
        height = 5760//width
        if acti_func == 'relu':
            self.activation = nn.ReLU(inplace=True)
        elif acti_func == 'tanh':
            self.activation = nn.Tanh()
        elif acti_func == 'leaky_relu':
            self.activation = nn.LeakyReLU(negative_slope=0.01)

        self.convs = nn.Sequential(
            nn.Conv2d(1, conv1_size, kernel_size=kernel1_size,
                      padding='same'),
            nn.Dropout(dropout),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(kernel_size=2),
            nn.Conv2d(conv1_size, conv2_size,
                      kernel_size=kernel2_size, padding='same'),
            nn.Dropout(dropout),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(kernel_size=2),
            nn.Conv2d(conv2_size, conv3_size,
                      kernel_size=kernel3_size, padding='same'),
            nn.Dropout(dropout),
        )
        self.cnn_size = int(conv3_size*(height//2//2)*(width//2//2))

        # This FC module dosen't work in the model prediction, but the absence of the module may cause some dropout issues.
        self.fc = nn.Sequential(
            nn.Linear(self.cnn_size, 256),
            self.activation,
            nn.Linear(256, 64),
            self.activation,
            nn.Linear(64, 1),
        )

    def forward(self, x):
        pass


class PL_CNN(pl.LightningModule):
    def __init__(self, config, total_steps, warmup_ratio=0.1, gpu=0):
        super().__init__()
        self.model = CNN_model(config)
        self.total_steps = total_steps
        self.warmup_ratio = warmup_ratio
        self.warmup_steps = math.ceil(total_steps * warmup_ratio)
        self.conv_type = config['conv_type']
        self.cnn_outsize = self.model.cnn_size
        self.final_fc = nn.Sequential(
            nn.Linear(self.cnn_outsize, 512),
            nn.ReLU(),
            nn.Linear(512, 256),
            nn.ReLU(),
            nn.Linear(256, 1)
        )
        self.config = config
        self.gpu = gpu
        self.batch_size = config['batch_size']
        self.acti_func = config['activation']
        self.train_acc = Accuracy()
        self.val_acc = Accuracy()
        self.save_hyperparameters(Namespace(**config))

    def forward(self, x):
        out = self.model.convs(x)
        out = out.flatten(1, -1)
        out = self.final_fc(out)
        out = torch.sigmoid(out)
        return out

    def get_cnn_feature(self, x):
        out = self.model.convs(x)
        out = out.flatten(1, -1)
        return out

    def get_loss(self, pred, label):
        loss = nn.BCELoss()
        return loss(pred, label.float())

    def training_step(self, batch, batch_idx):
        input, label = batch
        pred = self(input).squeeze(-1)
        loss = self.get_loss(pred, label)
        self.log(f'loss/train_loss', loss)
        self.train_acc(pred, label.int())
        self.log(f'acc/train_acc', self.train_acc,
                 on_step=False, on_epoch=True)
        return {'loss': loss}

    def validation_step(self, batch, batch_idx):
        input, label = batch
        pred = self(input).squeeze(-1)
        loss = self.get_loss(pred, label)
        self.val_acc(pred, label.int())
        self.log('acc/val_acc', self.val_acc, on_step=False, on_epoch=True)
        return {'val_loss': loss}

    def validation_epoch_end(self, outputs):
        avg_loss = torch.stack([x["val_loss"] for x in outputs]).mean()
        self.log(f'loss/val_loss', avg_loss)

    def configure_optimizers(self):
        optimizer = torch.optim.AdamW(
            self.parameters(), lr=1e-3, weight_decay=1e-2)
        scheduler = get_linear_schedule_with_warmup(
            optimizer, num_warmup_steps=self.warmup_steps, num_training_steps=self.total_steps)
        return {'optimizer': optimizer, 'lr_scheduler': scheduler}



import torch
import torch.nn as nn
import torch.nn.functional as F
import pytorch_lightning as pl
import math
import warnings
from torchmetrics import Accuracy
from .rnn_model import GRU_model
from .cnn_model import PL_CNN
from .utils import get_linear_schedule_with_warmup
from argparse import Namespace
warnings.filterwarnings(action='ignore')


class Fusion_model(nn.Module):
    def __init__(self, config, fusion_type, total_steps, use_weight=False, gpu=0):
        super().__init__()
        self.gpu = gpu
        self.use_weight = use_weight
        self.fusion_type = fusion_type
        self.config = config
        acti_func = config['activation']
        self.dropout = config['dropout']
        self.conv_type = config['conv_type']
        if self.use_weight:
            self.cnn_weight = nn.Parameter(torch.tensor(
                config['cnn_weight']), requires_grad=True)
            self.rnn_weight = nn.Parameter(torch.tensor(
                config['rnn_weight']), requires_grad=True)

        self.cnn = PL_CNN(config, total_steps=total_steps)
        self.rnn = GRU_model(config=config, dropout=self.dropout,
                             bidirectional=True, use_attention=True)

        if acti_func == 'relu':
            self.activation = nn.ReLU()
        elif acti_func == 'tanh':
            self.activation = nn.Tanh()
        elif acti_func == 'leaky_relu':
            self.activation = nn.LeakyReLU(negative_slope=0.01)
        self.indiv_fc_size = config['final_fc']

        self.cnn_fc = nn.Sequential(
            nn.Linear(self.cnn.cnn_outsize, self.indiv_fc_size), self.activation)
        self.rnn_fc = nn.Sequential(
            nn.Linear(self.rnn.rnn_outsize, self.indiv_fc_size),
            self.activation
        )
        self.fc = nn.Sequential(
            nn.Linear(self.indiv_fc_size, 512),
            self.activation,
            nn.Linear(512, 512),
            self.activation,
            nn.Linear(512, 256),
            self.activation,
            nn.Linear(256, 1))

    def forward(self, cnn_input, rnn_input):
        cnn_features = self.cnn.get_cnn_feature(cnn_input)
        rnn_features = self.rnn.get_rnn_feature(rnn_input)
        if self.use_weight:
            cnn_features = self.cnn_weight * cnn_features
            rnn_features = self.rnn_weight * rnn_features
        self.cnn_fc_out = F.dropout(self.cnn_fc(cnn_features), self.dropout)
        self.rnn_fc_out = F.dropout(self.rnn_fc(rnn_features), self.dropout)
        out = self.cnn_fc_out + self.rnn_fc_out
        out = self.fc(out)
        out = torch.sigmoid(out)
        return out


class  PL_fusion(pl.LightningModule):
    def __init__(self, config, total_steps, warmup_ratio=0.1, fusion_type='backbone', use_weight=False, gpu=0):
        super().__init__()
        self.fusion_type = fusion_type
        self.gpu = gpu
        self.total_steps = total_steps
        self.warmup_ratio = warmup_ratio
        self.warmup_steps = math.ceil(total_steps * warmup_ratio)
        self.model = Fusion_model(
            config, fusion_type, total_steps, use_weight, gpu)
        self.config = config
        self.batch_size = config['batch_size']
        self.acti_func = config['activation']
        self.train_acc = Accuracy()
        self.val_acc = Accuracy()
        self.save_hyperparameters(Namespace(**config))

    def forward(self, cnn_input, rnn_input):
        return self.model(cnn_input, rnn_input)

    def get_loss(self, pred, label):
        loss = nn.BCELoss()
        return loss(pred, label.float())

    def training_step(self, batch, batch_idx):
        cnn_input = batch['cnn_input']
        rnn_input = batch['rnn_input']
        label = batch['label']
        pred = self.forward(cnn_input=cnn_input,
                            rnn_input=rnn_input).squeeze()
        loss = self.get_loss(pred, label)
        self.log(f'loss/train_loss', loss)
        self.train_acc(pred, label.int())
        self.log(f'acc/train_acc', self.train_acc,
                 on_step=False, on_epoch=True)
        opt = self.optimizers()
        self.log('lr', opt.state_dict()['param_groups'][0]['lr'])
        return {'loss': loss}

    def validation_step(self, batch, batch_idx):
        cnn_input = batch['cnn_input']
        rnn_input = batch['rnn_input']
        label = batch['label']
        pred = self.forward(cnn_input=cnn_input,
                            rnn_input=rnn_input).squeeze(-1)
        loss = self.get_loss(pred, label)
        self.val_acc(pred, label.int())
        self.log('acc/val_acc', self.val_acc, on_step=False, on_epoch=True)
        return {'val_loss': loss}

    def validation_epoch_end(self, outputs):
        avg_loss = torch.stack([x["val_loss"] for x in outputs]).mean()
        self.log(f'loss/val_loss', avg_loss)

    def configure_optimizers(self):
        optimizer = torch.optim.AdamW(
            self.parameters(), lr=1e-3, weight_decay=1e-2)
        scheduler = get_linear_schedule_with_warmup(
            optimizer, num_warmup_steps=self.warmup_steps, num_training_steps=self.total_steps)
        return {'optimizer': optimizer, 'lr_scheduler': scheduler}



#!/usr/bin/python

#       Sgourakis Lab
#   Author: Santrupti Nerli
#   Date: May 25, 2018
#   Email: snerli@ucsc.edu
#

'''

ARGPARSE class contains all the necessary functionalities required to accept input from
users.

'''

import argparse


class ARGPARSE:

    # class member
    args = None # stores all the arguments

    # constructor
    def __init__(self):
        self.parse_args()
        self.check_user_input()

    # method that provides parser arguments that are seen on the command line when
    # running this method
    def parse_args(self):
        parser = argparse.ArgumentParser(description='Perform threading of the template structure onto the target sequence')
        parser.add_argument("-list_mhcs", help="List all the HLAs for which sequences are available in the database", action='store_true')
        parser.add_argument("-template_pdb", help="provide template structure in PDB to perform threading")
        parser.add_argument("-mhcs", help="provide the list of names of MHCs in the file, if you want to include all, just type \'all\' in the file")
        parser.add_argument("-beta2m", help="provide the file containing names of beta2m, Choices include: choices=[humanbeta2m, mousebeta2m, chickenbeta2m, bovinebeta2m, ratbeta2m, none]")
        parser.add_argument("-peptides", help="provide fasta file with peptide sequences that need to be threaded")
        parser.add_argument("-tcr", help="provide the file containing tcr sequence in fasta format")
        parser.add_argument("-chaperone", help="provide the file containing names of chaperones, Choices include: choices=[tapasin, tapbpr, none]")
        parser.add_argument("-mhc_trim_length", help="provide the number of residue from which the mhc should be trimmed", type=int, default=181)
        parser.add_argument("-no_trim_mhc", help="Should we model the whole complex", action='store_false')
        parser.add_argument("-idealize_relax", help="idealize and relax template structure before threading", action='store_true')
        parser.add_argument("-relax_after_threading", help="idealize and relax template structure before threading", action='store_true')
        parser.add_argument("-mhc_chain", help="provide mhc chain id in the template")
        parser.add_argument("-peptide_chain", help="provide peptide chain id in the template")
        parser.add_argument("-pep_start_index", help="provide peptide start index", type=int, default=0)
        parser.add_argument("-groove_distance", help="provide distance to select nearest groove residues from the peptide", type=float, default=3.5)
        parser.add_argument("-interface_cutpoint", help="last residue index that separates the interfaces for which you are calculating binding energies", type=int, default=0)
        parser.add_argument("-out_file", help="output file name in csv format to write the binding energies", default="binding_energies.csv")
        parser.add_argument("-nstruct", help="number of times a threaded structure should be relaxed", type=int, default=1)
        parser.add_argument("-clustal_path", help="Path to clustal omega", default='clustalo')
        parser.add_argument("-native", help="native pdb file to compare and report RMSD values", default=None)

        self.args = parser.parse_args()

    # method to validate user specified input
    # check if a more extensive check is required
    def check_user_input(self):
        if self.is_list_mhcs() == False:
            if self.get_mhcs() == None or self.get_template_pdb() == None:
                print("Please provide mhc list, peptide list and template_pdb to perform threading\n Type -h to see all the options")
                exit(1)

    # methods that store actions
    def is_list_mhcs(self):
        return self.args.list_mhcs

    def is_no_trim_mhc_flag_set(self):
        return self.args.no_trim_mhc

    # getter methods that return all user provided inputs
    def get_template_pdb(self):
        return self.args.template_pdb

    def get_interface_cupoint(self):
        return self.args.interface_cutpoint

    def get_mhc_trim_length(self):
        return self.args.mhc_trim_length

    def get_mhcs(self):
        return self.args.mhcs

    def get_peptides(self):
        return self.args.peptides

    def get_groove_distance(self):
        return self.args.groove_distance

    def get_nstruct(self):
        return self.args.nstruct

    def get_idealize_relax(self):
        return self.args.idealize_relax

    def get_relax_after_threading(self):
        return self.args.relax_after_threading

    def get_beta2m(self):
        if self.is_no_trim_mhc_flag_set():
            return None
        return self.args.beta2m

    def get_tcr(self):
        return self.args.tcr

    def get_chaperone(self):
        if self.is_no_trim_mhc_flag_set():
            return None
        return self.args.chaperone

    def get_peptide_chain(self):
        return self.args.peptide_chain

    def get_mhc_chain(self):
        return self.args.mhc_chain

    def get_pep_start_index(self):
        return self.args.pep_start_index

    def get_out_file(self):
        return self.args.out_file

    def get_clustal_path(self):
        return self.args.clustal_path

    def get_native(self):
        return self.args.native



# additional bio libraries

from pyrosetta import *

#custom libraries
from chain_split import CHAIN_SPLIT
from interface_analyzer import INTERFACE

# import other required libraries
import os
import sys
import argparse

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='call ia')
    parser.add_argument("-dir", help="provide directory containing pdb names")
    parser.add_argument("-cutpoint", help="provide cutpoint", type=int)
    parser.add_argument("-pattern", help="provide pattern to look for in the file name", default="relaxed")
    parser.add_argument("-outfile", help="provide output file name")
    args = parser.parse_args()

    init()

    writefilehandler = open(args.outfile,"w")

    for filename in os.listdir(args.dir):
        if args.pattern in filename and ".pdb" in filename:
            my_pose = pose_from_pdb(args.dir+"/"+filename)
            chain = CHAIN_SPLIT(my_pose, args.cutpoint)
            chain.cut()
            split = chain.get_pose()

            ia = INTERFACE(split)
            ia.analyze()
            print("Interface energy: "+str(ia.get_dG()))
            writefilehandler.write(filename+" "+str(ia.get_dG()))

    writefilehandler.close()



# additional bio libraries

from pyrosetta import *

#custom libraries
from rmsd import RMSD

# import other required libraries
import os
import sys
import argparse

def extract_residues_from_movemap(movemap):
    readfilehandler = open(movemap, "r")
    resi_list = pyrosetta.rosetta.std.list_unsigned_long_t()
    for line in readfilehandler:
        line = line.rstrip()
        if "RESIDUE" in line and "CHI" in line:
            resi = line.split(" ")[1]
            resi_list.append(int(resi))
    readfilehandler.close()
    return resi_list

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='compute rmsd values')
    parser.add_argument("-dir", help="provide directory containing pdb files")
    parser.add_argument("-native", help="provide native pdb file")
    parser.add_argument("-pattern", help="provide pattern to look for in the target file name")
    parser.add_argument("-movemap", help="find rmsd value for the residues used in the movemap")
    parser.add_argument("-choice", help="which type of rmsd do you want to calculate", choices=['all','CA','CA_with_start_and_end','CA_with_movemap'])
    parser.add_argument("-start", help="start residue index", type=int)
    parser.add_argument("-end", help="end residue index", type=int)
    parser.add_argument("-outfile", help="provide output file name", default="rmsds.txt")
    args = parser.parse_args()

    init()

    writefilehandler = open(args.outfile,"w")

    native_pose = pose_from_pdb(args.native)

    for filename in os.listdir(args.dir):
        if args.pattern in filename and ".pdb" in filename:
            rms_value = 0
            my_pose = pose_from_pdb(args.dir+"/"+filename)
            rms_obj = RMSD(my_pose, native_pose)

            if args.choice == "all":
                rms_value = rms_obj.all_atom_rmsd()
            elif args.choice == "CA":
                rms_value = rms_obj.CA_rmsd()
            elif args.choice == "CA_with_start_and_end":
                rms_value = rms_obj.CA_rmsd_between_indices(start, end)
            elif args.choice == "CA_with_movemap":
                resi_list = extract_residues_from_movemap(args.movemap)
                rms_value = rms_obj.CA_rmsd_residue_number(resi_list)
            writefilehandler.write(filename+" "+str(rms_value)+"\n")

    writefilehandler.close()



from subprocess import call
import sys

atoms = []
connects = []

residue_atom_names = {"ARG":['CA', 'C', 'CB', 'CG', 'CD', 'NE', 'CZ', 'NH1', 'NH2'],
                      "LYS":['CA', 'C', 'CB', 'CG', 'CD', 'CE', 'NZ'],
                      "ASP":['CA', 'C', 'CB', 'CG', 'OD1', 'OD2'],
                      "GLU":['CA', 'C', 'CB', 'CG', 'CD', 'OE1', 'OE2'],
                      "SER":['CA', 'C', 'CB', 'OG'],
                      "CYS":['CA', 'C', 'CB', 'SG'],
                      "GLY":['CA', 'C'],
                      "ALA":['CA', 'C', 'CB'],
                      "VAL":['CA', 'C', 'CB', 'CG1', 'CG2'],
                      "LEU":['CA', 'C', 'CB', 'CG', 'CD1', 'CD2'],
                      "MET":['CA', 'C', 'CB', 'CG', 'SD', 'CE'],
                      }


new_file_name = sys.argv[1] + ".temp" #"receptor_part.pdb"
new_connect_file_name = sys.argv[1] + ".connect" #"temp_conect"
call(["cp " + sys.argv[1] + " " + new_file_name], shell=True)
call(["grep \"CONECT\" " + new_file_name + " > " + new_connect_file_name], shell=True)

conect_lines = []
f = open(new_connect_file_name, 'r')
for line in f: conect_lines.append(line)
f.close()

prev_resi_num = -1
f = open(sys.argv[1], 'r')
residues = []
residues_conect = []
index_start = 0
residue = []
for line in f:

	line_arr = line.split()
	if line_arr[0] != "ATOM": continue
	resi_num = line_arr[5]
	
	if resi_num != prev_resi_num and len(residue) > 0: # new residue
		residues.append(residue)
		residues_conect.append(conect_lines[index_start:(index_start + len(residue))])
		index_start += len(residue)
		residue = []

	residue.append(line)
	prev_resi_num = resi_num


f.close()

for residue_index,residue in enumerate(residues):

	curr_residue_name = residue[0][17:20]
	curr_conect_line = residues_conect[residue_index]

	if curr_residue_name in residue_atom_names.keys():

		name_index = 0
		for line in residue:
			old_line = line[:11] + "  " + line[13:20]

			new_atom_name = residue_atom_names[curr_residue_name][name_index]
			space_to_residue_name = -1
			if len(new_atom_name) == 1:   space_to_residue_name = "   "
			elif len(new_atom_name) == 2: space_to_residue_name = "  "
			elif len(new_atom_name) == 3: space_to_residue_name = " "
			new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

			call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)
			name_index += 1

	else:

		if curr_residue_name == "THR":

			atom_names = ['CA', 'C', 'CB', 'CG2']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				if line_arr[2] == "O":
					new_atom_name = "OG1"
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)

		elif curr_residue_name == "ASN":

			atom_names = ['CA', 'C', 'CB', 'CG']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				if line_arr[2] == "O":
					new_atom_name = "OD1"
				elif line_arr[2] == "N":
					new_atom_name = "ND2"
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)

		elif curr_residue_name == "GLN":

			atom_names = ['CA', 'C', 'CB', 'CG', 'CD']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				if line_arr[2] == "O":
					new_atom_name = "OE1"
				elif line_arr[2] == "N":
					new_atom_name = "NE2"
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)

		elif curr_residue_name == "ILE":

			CB_line = residue[2] # CA, C, then CB
			CB_line_arr = CB_line.split()
			CB_num = CB_line_arr[1] # first word is CONECT

			unknown_lines = curr_conect_line[3:]
			atom_mapping = {}

			# the atom that only has a single connection to CB_num is CG2
			# the atom that has two connections, one of them to CB_num, is CG1
			# the atom that has a single connection, not to CB_num, is CD1
			for line in unknown_lines:
				line_arr = line.split()
				curr_atom = line_arr[1]
				connected_atoms = line_arr[2:]
				if len(connected_atoms) == 1 and CB_num in connected_atoms: atom_mapping[curr_atom] = "CG2"
				elif len(connected_atoms) > 1: atom_mapping[curr_atom] = "CG1"
				elif len(connected_atoms) == 1 and CB_num not in connected_atoms: atom_mapping[curr_atom] = "CD1"


			atom_names = ['CA', 'C', 'CB']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				curr_atom = line_arr[1]

				if curr_atom in atom_mapping.keys():
					new_atom_name = atom_mapping[curr_atom]
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1	

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)

		elif curr_residue_name == "HIS":

			CG_line = residue[3] # CA, C, CB, then CG
			CG_line_arr = CG_line.split()
			CG_num = CG_line_arr[1] # first word is CONECT

			unknown_lines_beg = 4
			atom_mapping = {}

			# finding the N atom that connects to CG_num must be ND1
			# finding the C atom that connects to CG_num must be CD2
			# finding the C atom remaining that connects to ND1 must be CE1
			# finding the N atom remaining that connects to CE1 must be NE2
			for mode in range(4):
				for line_index in range(unknown_lines_beg, len(residue)):
					c_line = curr_conect_line[line_index]
					a_line = residue[line_index]
					a_line_arr = a_line.split()
					curr_atom = a_line_arr[1]
					elem = a_line_arr[2]
					c_line_arr = c_line.split()
					connected_atoms = c_line_arr[2:]

					if mode == 0:
						if elem == 'N' and CG_num in connected_atoms: atom_mapping[curr_atom] = "ND1" 
					elif mode == 1:
						if elem == 'C' and CG_num in connected_atoms: atom_mapping[curr_atom] = "CD2"
					elif mode == 2:
						if elem == 'C' and curr_atom not in atom_mapping.keys():
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'ND1': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'ND1': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CE1"
					elif mode == 3:
						if elem == 'N' and curr_atom not in atom_mapping.keys():
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CE1': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CE1': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "NE2"


			atom_names = ['CA', 'C', 'CB', 'CG']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				curr_atom = line_arr[1]

				if curr_atom in atom_mapping.keys():
					new_atom_name = atom_mapping[curr_atom]
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1	

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)

		elif curr_residue_name == "PRO":

			atom_names = ['CA', 'C', 'CB', 'CG', 'CD']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				if line_arr[2] == "N":
					new_atom_name = "N"
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)

		elif curr_residue_name == "PHE":

			CD1_line = residue[4] # CA, C, CB, CG, then CD1
			CD1_line_arr = CD1_line.split()
			CD1_num = CD1_line_arr[1] # first word is CONECT

			unknown_lines_beg = 5
			atom_mapping = {}

			# finding the C atom that connects to CD1_num must be CE1
			# finding the C atom remaining that connects to CE1 must be CZ
			# finding the C atom remaining that connects to CZ must be CE2
			# finding the C atom remaining that connects to CE2 must be CD2
			for mode in range(4):
				for line_index in range(unknown_lines_beg, len(residue)):
					c_line = curr_conect_line[line_index]
					a_line = residue[line_index]
					a_line_arr = a_line.split()
					curr_atom = a_line_arr[1]
					elem = a_line_arr[2]
					c_line_arr = c_line.split()
					connected_atoms = c_line_arr[2:]

					if mode == 0:
						if elem == 'C' and CD1_num in connected_atoms: atom_mapping[curr_atom] = "CE1" 
					elif mode == 1:
						if elem == 'C' and curr_atom not in atom_mapping.keys():
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CE1': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CE1': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CZ"
					elif mode == 2:
						if elem == 'C' and curr_atom not in atom_mapping.keys():
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CZ': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CZ': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CE2"
					elif mode == 3:
						if elem == 'C' and curr_atom not in atom_mapping.keys():
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CE2': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CE2': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CD2"


			atom_names = ['CA', 'C', 'CB', 'CG', 'CD1']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				curr_atom = line_arr[1]

				if curr_atom in atom_mapping.keys():
					new_atom_name = atom_mapping[curr_atom]
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1	

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)
				
		elif curr_residue_name == "TYR":

			CD1_line = residue[4] # CA, C, CB, CG, then CD1
			CD1_line_arr = CD1_line.split()
			CD1_num = CD1_line_arr[1] # first word is CONECT

			unknown_lines_beg = 5
			atom_mapping = {}

			# finding the C atom that connects to CD1_num must be CE1
			# finding the C atom remaining that has three connections must be CZ
			# the O atom must be OH
			# finding the C atom remaining that connects to CZ must be CE2
			# finding the C atom remaining that connects to CE2 must be CD2
			for mode in range(5):
				for line_index in range(unknown_lines_beg, len(residue)):
					c_line = curr_conect_line[line_index]
					a_line = residue[line_index]
					a_line_arr = a_line.split()
					curr_atom = a_line_arr[1]
					elem = a_line_arr[2]
					c_line_arr = c_line.split()
					connected_atoms = c_line_arr[2:]

					if mode == 0:
						if elem == 'C' and CD1_num in connected_atoms: atom_mapping[curr_atom] = "CE1" 
					elif mode == 1:
						if elem == 'C' and len(connected_atoms) == 3: atom_mapping[curr_atom] = "CZ"
					elif mode == 2:
						if elem == 'O': atom_mapping[curr_atom] = "OH"
					elif mode == 3:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 2:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CZ': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CZ': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CE2"
					elif mode == 4:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 2:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CE2': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CE2': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CD2"


			atom_names = ['CA', 'C', 'CB', 'CG', 'CD1']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				curr_atom = line_arr[1]

				if curr_atom in atom_mapping.keys():
					new_atom_name = atom_mapping[curr_atom]
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1	

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)


		elif curr_residue_name == "TRP":

			CG_line = residue[3] # CA, C, CB, then CG
			CG_line_arr = CG_line.split()
			CG_num = CG_line_arr[1] # first word is CONECT

			unknown_lines_beg = 4
			atom_mapping = {}

			# 0 the N atom is NE1
			# 1 the C atom that connects to CG_num and connects to NE1 must be CD1
			# 2 the C atom remaining that has three connections and connects to NE1 must be CE2
			# 3 the C atom remaining that has three connections and connects to CE2 must be CD2
			# 4 the C atom remaining that connects to CD2 must be CE3
			# 5 the C atom reminaing that connects to CE3 must be CZ3
			# 6 the C atom remaining that connects to CZ3 must be CH2
			# 7 the C atom remaining that connects to CH2 must be CZ2

			for mode in range(8):
				for line_index in range(unknown_lines_beg, len(residue)):
					c_line = curr_conect_line[line_index]
					a_line = residue[line_index]
					a_line_arr = a_line.split()
					curr_atom = a_line_arr[1]
					elem = a_line_arr[2]
					c_line_arr = c_line.split()
					connected_atoms = c_line_arr[2:]

					if mode == 0:
						if elem == 'N': atom_mapping[curr_atom] = "NE1" 
					elif mode == 1:
						if elem == 'C' and CG_num in connected_atoms and len(connected_atoms) == 2: atom_mapping[curr_atom] = "CD1"
					elif mode == 2:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 3:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'NE1': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'NE1': a1 = True
							a2 = False
							if connected_atoms[2] in atom_mapping.keys():
								if atom_mapping[connected_atoms[2]] == 'NE1': a2 = True
							if a0 or a1 or a2: atom_mapping[curr_atom] = "CE2"
					elif mode == 3:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 3:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CE2': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CE2': a1 = True
							a2 = False
							if connected_atoms[2] in atom_mapping.keys():
								if atom_mapping[connected_atoms[2]] == 'CE2': a2 = True
							if a0 or a1 or a2: atom_mapping[curr_atom] = "CD2"
					elif mode == 4:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 2:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CD2': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CD2': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CE3"
					elif mode == 5:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 2:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CE3': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CE3': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CZ3"
					elif mode == 6:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 2:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CZ3': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CZ3': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CH2"
					elif mode == 7:
						if elem == 'C' and curr_atom not in atom_mapping.keys() and len(connected_atoms) == 2:
							a0 = False
							if connected_atoms[0] in atom_mapping.keys():
								if atom_mapping[connected_atoms[0]] == 'CH2': a0 = True
							a1 = False
							if connected_atoms[1] in atom_mapping.keys():
								if atom_mapping[connected_atoms[1]] == 'CH2': a1 = True
							if a0 or a1: atom_mapping[curr_atom] = "CZ2"


			atom_names = ['CA', 'C', 'CB', 'CG']
			name_index = 0
			for line in residue:
				old_line = line[:11] + "  " + line[13:20]

				line_arr = line.split()
				curr_atom = line_arr[1]

				if curr_atom in atom_mapping.keys():
					new_atom_name = atom_mapping[curr_atom]
				else: 
					new_atom_name = atom_names[name_index]
					name_index += 1	

				space_to_residue_name = -1
				if len(new_atom_name) == 1:   space_to_residue_name = "   "
				elif len(new_atom_name) == 2: space_to_residue_name = "  "
				elif len(new_atom_name) == 3: space_to_residue_name = " "
				new_line = line[:11] + "  " + new_atom_name + space_to_residue_name + curr_residue_name

				call(["sed -i \"s/" + old_line + "/" + new_line + "/g\" " + new_file_name], shell=True)



call(["rm " + new_connect_file_name], shell=True)

import mdtraj as md

orig = md.load_pdb("receptor.pdb")

temp = md.load_pdb(new_file_name)
#orig_atoms = [a for a in orig.top.atoms]
temp_atoms = [a for a in temp.top.atoms]

for i, temp_atom in enumerate(temp_atoms):
    dash_index = str(temp_atom).find('-')
    residue_name = str(temp_atom)[:dash_index]
    element_type = str(temp_atom)[dash_index+1:]
    residue_index = int(residue_name[3:]) - 1
    
    orig_atoms_in_residue_indices = orig.top.select("resid == " + str(residue_index) + " and name == " + element_type)
    orig.xyz[0, orig_atoms_in_residue_indices[0], :] = temp.xyz[0, i, :]


orig.save_pdb(sys.argv[1] + ".complete")

sys.exit(0)








#!/usr/bin/python

#       Sgourakis Lab
#   Author: Santrupti Nerli
#   Date: December 1, 2017
#   Email: snerli@ucsc.edu
#

# Load the Rosetta commands for use in the Python shell
from pyrosetta import *

# import rosetta files
from pyrosetta.rosetta.protocols.comparative_modeling import *
from pyrosetta.rosetta.core.scoring import *
from pyrosetta.rosetta.core.sequence import *

#custom libraries
from idealize_relax.relax import RELAX
from thread.template import TEMPLATE
from thread.fasta import FASTA
from alignment.align import ALIGN
from alignment.grishin import GRISHIN

# import other required libraries
import os
import sys


'''

PRE_THREADING class contains all the necessary functionalities required to
perform operations before the threading process.

'''

class PRE_THREADING:

    # constructor
    template = None # store the template structure
    alignment = None # alignment object to store the alignment of template and target
    grishin_file_name = "" # grishin file name
    complex_header = "" # target complex header name
    complex_sequence = "" # target complex sequence
    pep_length = 0 # peptide length
    args = None # input arguments

    # constructor
    def __init__(self, template, complex_sequence, complex_header, args):
        self.template = template
        self.complex_sequence = complex_sequence
        self.complex_header = complex_header
        self.args = args
        self.pep_length = len(list(self.complex_header.split("_")[2]))
        self.align_template_target_sequences()

    # method to return target file name
    def get_target_file_name(self):
        return self.complex_header+"_on_"+self.template.get_stripped_name()

    # method to return target sequence
    def get_target_sequence(self):
        return self.complex_sequence

    # method to return peptide start index in the sequence
    def get_pep_start_index(self, sequence):
        index = 0
        try:
            index = sequence.rindex(self.peptide_sequence)
            return index
        except ValueError:
            print("Alignment failed. Cannot find peptide sequence")
            exit(1)

    # method to get the grishin file name
    def get_grishin_file_name(self):
        return self.grishin_file_name

    # method to return the template
    def get_template(self):
        return self.template

    # method to return the mhc header
    def get_mhc_header(self):
        return self.complex_header.split("_")[0]

    # method to return the peptide length
    def get_pep_length(self):
        return self.pep_length

    # method to check if grishin file exists
    def check_if_grishin_file_exists(self, filename):
        for f in self.grishin_file_name:
            if f == filename:
                return True

    # method to create a grishin file
    def create_grishin(self, aligned_target, aligned_query, is_new = False):
        # template_seq = query_sequence , target_seq =  target_sequence
        grishin = GRISHIN(self.get_target_file_name(), self.complex_header.split("_")[0], self.template.get_name(),
                            aligned_target, aligned_query)

        if not self.check_if_grishin_file_exists(grishin.get_file_name()):
            self.grishin_file_name = grishin.get_file_name()
            grishin.write()

    # method to align template and target sequences using clustal
    def align_template_target_sequences(self):
        key = self.complex_header.split("_")[0]
        self.alignment = ALIGN(self.template.get_sequence(), self.complex_header, self.complex_sequence, self.args.get_clustal_path())
        self.alignment.clustal()
        alignment = FASTA(self.alignment.get_clustal_output_filename())
        alignment.read()
        aln_seqs = alignment.get_sequences()

        for key,value in aln_seqs.items():
            if key != 'template':
                self.create_grishin(value, aln_seqs['template'])
